function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

import { NativeModules, NativeEventEmitter } from 'react-native';
const {
  GoogleCloudSpeechToText
} = NativeModules;
const VoiceEmitter = new NativeEventEmitter(GoogleCloudSpeechToText);
var ChannelCount;

(function (ChannelCount) {
  ChannelCount[ChannelCount["MONO"] = 1] = "MONO";
  ChannelCount[ChannelCount["STEREO"] = 2] = "STEREO";
})(ChannelCount || (ChannelCount = {}));

class GCSpeechToText {
  constructor() {
    _defineProperty(this, "_events", void 0);

    _defineProperty(this, "_listeners", void 0);

    this._listeners = null;
    this._events = {
      onVoice: () => undefined,
      onVoiceEnd: () => undefined,
      onSpeechError: () => undefined,
      onVoiceStart: () => undefined,
      onSpeechRecognized: () => undefined,
      onSpeechRecognizing: () => undefined
    };
  }
  /**
   * Start speech recognize
   * return file Id: number if set saveToFile = true
   * @param options
   */


  async start(options) {
    if (!this._listeners) {
      this._listeners = Object.keys(this._events).map(key => VoiceEmitter.addListener(key, this._events[key]));
    }

    return await GoogleCloudSpeechToText.start(Object.assign({
      languageCode: 'en-US',
      speechToFile: false
    }, options));
  }

  async stop() {
    return await GoogleCloudSpeechToText.stop();
  }

  setApiKey(apiKey) {
    GoogleCloudSpeechToText.setApiKey(apiKey);
  }
  /**
   * get recognized voice as aac file
   * @param fileId
   * @param options
   */


  async getAudioFile(fileId, options) {
    return await GoogleCloudSpeechToText.getAudioFile(fileId, Object.assign({
      channel: ChannelCount.STEREO,
      sampleRate: 44100,
      bitrate: 96000
    }, options));
  }

  removeListeners() {
    if (this._listeners) {
      this._listeners.map(listener => listener.remove());

      this._listeners = null;
    }

    this._listeners = null;
  }

  async destroy() {
    this.removeListeners();
    await GoogleCloudSpeechToText.destroy();
  }

  onVoiceStart(fn) {
    this._events.onVoiceStart = fn;
  }

  onVoice(fn) {
    this._events.onVoice = fn;
  }

  onVoiceEnd(fn) {
    this._events.onVoiceEnd = fn;
  }

  onSpeechError(fn) {
    this._events.onSpeechError = fn;
  }

  onSpeechRecognized(fn) {
    this._events.onSpeechRecognized = fn;
  }

  onSpeechRecognizing(fn) {
    this._events.onSpeechRecognizing = fn;
  }

}

export default new GCSpeechToText();
//# sourceMappingURL=index.js.map