declare enum ChannelCount {
    MONO = 1,
    STEREO = 2
}
declare type SampleRate = 16000 | 11025 | 22050 | 44100;
declare type Bitrate = 64000 | 96000 | 128000 | 192000 | 256000;
export interface SpeechEvents {
    onVoiceStart?: (e: VoiceStartEvent) => void;
    onVoice?: (e: VoiceEvent) => void;
    onVoiceEnd?: () => void;
    onSpeechError?: (e: SpeechErrorEvent) => void;
    onSpeechRecognized?: (e: SpeechRecognizeEvent) => void;
    onSpeechRecognizing?: (e: SpeechRecognizeEvent) => void;
}
export interface VoiceStartEvent {
    sampleRate: number;
    voiceRecorderState: number;
}
export interface VoiceEvent {
    size: number;
}
export interface SpeechErrorEvent {
    error?: {
        code?: string;
        message?: string;
    };
}
export interface SpeechRecognizeEvent {
    isFinal: boolean;
    transcript: string;
}
export interface SpeechStartEvent {
    fileId: string;
    tmpPath: string;
}
export interface StartOptions {
    speechToFile?: boolean;
    languageCode?: string;
}
export interface OutputFile {
    size: number;
    path: string;
}
export interface OutputConfig {
    sampleRate?: SampleRate;
    bitrate?: Bitrate;
    channel?: ChannelCount;
}
declare class GCSpeechToText {
    private readonly _events;
    private _listeners;
    constructor();
    /**
     * Start speech recognize
     * return file Id: number if set saveToFile = true
     * @param options
     */
    start(options?: StartOptions): Promise<SpeechStartEvent>;
    stop(): Promise<void>;
    setApiKey(apiKey: string): void;
    /**
     * get recognized voice as aac file
     * @param fileId
     * @param options
     */
    getAudioFile(fileId: string, options?: OutputConfig): Promise<OutputFile>;
    removeListeners(): void;
    destroy(): Promise<void>;
    onVoiceStart(fn: (data: VoiceStartEvent) => void): void;
    onVoice(fn: (data: VoiceEvent) => void): void;
    onVoiceEnd(fn: () => void): void;
    onSpeechError(fn: (error: SpeechErrorEvent) => void): void;
    onSpeechRecognized(fn: (data: SpeechRecognizeEvent) => void): void;
    onSpeechRecognizing(fn: (data: SpeechRecognizeEvent) => void): void;
}
declare const _default: GCSpeechToText;
export default _default;
